@@ -43,7 +43,9 @@ CONFIG_DEFAULT_VALUE = {
     "TOKEN": None,
     # If the token is an org token, default app_name
     "APP_NAME": None,
-    # Proxy to use if needed
+    # Proxy to use if needed, takes precedence over PROXY_URL
+    "HTTP_PROXY": None,
+    # Proxy to use if needed, legacy name
     "PROXY_URL": None,
     # Should the client check the rules signatures
     "RULES_SIGNATURE": True,
@@ -74,6 +76,11 @@ CONFIG_DEFAULT_VALUE = {
     "GRACEFUL_TIMEOUT": 5.0,
     # Static instrumentation, enabled by default
     "BETA_STATIC_INSTRUMENTATION": True,
+    # Use the Heroku DYNO environement variable as hostname
+    "USE_DYNO_HOSTNAME": True,
+    # Max length of Sqreen queues
+    "MAX_QUEUE_LENGTH": 100,
+    "MAX_OBS_QUEUE_LENGTH": 10000,
 }
 
 
@@ -177,7 +184,7 @@ class Config(object):
             value = value.lower().strip() in ("1", "true", "yes", "y")
         elif isinstance(default_value, (int, float)):
             try:
-                value = float(value)
+                value = type(default_value)(value)
             except ValueError:
                 LOGGER.error(
                     "Invalid config value for %s, using default value",
