@@ -7,8 +7,15 @@
 """ Simple delivery method that directly call session on event
 """
 
+from .._vendors.sqreen_security_signal_sdk.compat_model import Signal
 from ..events import Attack, RequestRecord
 from ..remote_exception import RemoteException
+from ..utils import HAS_TYPING
+
+if HAS_TYPING:
+    from typing import Mapping, Optional, Sequence, Union
+
+    from ..session import Session
 
 
 class SimpleDeliverer(object):
@@ -21,22 +28,33 @@ class SimpleDeliverer(object):
     max_staleness = 0
 
     def __init__(self, session):
+        # type: (Session) -> None
         self.session = session
 
     def post_event(self, event):
+        # type: (Union[RemoteException, Attack, RequestRecord, Signal]) -> Optional[Mapping]
         """ Post a single event
         """
         if isinstance(event, RemoteException):
-            return self.session.post_sqreen_exception(event.to_dict())
+            return self.session.post_sqreen_exception(event)
         if isinstance(event, Attack):
-            return self.session.post_attack(event.to_dict())
+            return self.session.post_attack(event)
         if isinstance(event, RequestRecord):
-            return self.session.post_request_record(event.to_dict())
+            return self.session.post_request_record(event)
+        if isinstance(event, dict):
+            return self.session.post_signal(event)
         else:
             err_msg = "Unknown event type {}".format(type(event))
             raise NotImplementedError(err_msg)
 
+    def post_metrics(self, metrics):
+        # type: (Sequence[Mapping]) -> Optional[Mapping]
+        """ Post metrics
+        """
+        return self.session.post_metrics(metrics)
+
     def drain(self, resiliently):
+        # type: (bool) -> None
         """ Since everything is posted at once nothing needs to be done here
         """
         pass
