@@ -11,7 +11,7 @@ class ClientTracer:
     def format_payload(self, scope, transport):
         if scope == "client":
             return {
-                "transport": "http",
+                "transport": transport.get("type"),
                 "host": transport.get("host"),
                 "ip": transport.get("host_ip"),
                 "port": transport.get("host_port"),
