@@ -72,7 +72,7 @@ class RequestRecord(object):
     def to_dict(self):
         """Export the record as a dict object."""
         result = {"version": self.VERSION}
-        sensitive_values = None
+        sensitive_values = False
 
         if "request" in self.payload:
             request_dict = self.payload["request"]
@@ -101,7 +101,7 @@ class RequestRecord(object):
             for attack_dict in attacks:
                 rulespack = attack_dict.pop("rulespack_id", None) or rulespack
             if attacks and sensitive_values:
-                observed_dict["attacks"] = list(sanitize_attacks(attacks, sensitive_values))
+                observed_dict["attacks"] = list(sanitize_attacks(attacks))
             exceptions = observed_dict.get("sqreen_exceptions", [])
             for exc_dict in exceptions:
                 payload_dict = exc_dict.pop("exception", None)
@@ -121,10 +121,19 @@ class RequestRecord(object):
                     ]
                 ]
             if "sdk" in observed_dict:
-                result["observed"]["sdk"] = [
-                    {"name": entry[0], "time": entry[1], "args": entry[2:]}
-                    for entry in observed_dict["sdk"]
-                ]
+                global_user_identifiers = None
+                for entry in observed_dict["sdk"]:
+                    if entry[0] == "identify":
+                        global_user_identifiers = entry[2]
+                        LOGGER.debug("sdk identifiers: %r", global_user_identifiers)
+                        break
+                sdk_events = []
+                for entry in observed_dict["sdk"]:
+                    if global_user_identifiers is not None and entry[0] == "track":
+                        entry[3].setdefault("user_identifiers", global_user_identifiers)
+                    sdk_events.append({"name": entry[0], "time": entry[1], "args": entry[2:]})
+                result["observed"]["sdk"] = sdk_events
+
         if "local" in self.payload:
             result["local"] = self.payload["local"]
         return result
