@@ -6,9 +6,11 @@
 #
 """ Pyramid specific WSGI HTTP Request / Response stuff
 """
+from cgi import FieldStorage
+from copy import deepcopy
 from logging import getLogger
 
-from ..utils import is_json_serializable
+from ..utils import cached_property
 from .base import BaseResponse
 from .wsgi import BaseWSGIRequest
 
@@ -21,23 +23,15 @@ class PyramidRequest(BaseWSGIRequest):
         super(PyramidRequest, self).__init__(storage=storage)
         self.request = request
 
-        # Cache for params
-        self._query_params = None
-        self._form_params = None
-        self._cookies_params = None
-        self._json_params = None
-
-    @property
+    @cached_property
     def query_params(self):
-        if self._query_params is None:
-            try:
-                # Convert pyramid MultiDict to a normal dict with values as list
-                self._query_params = dict(self.request.GET.dict_of_lists())
-            except Exception:
-                LOGGER.debug("couldn't get request.GET from the framework",
-                             exc_info=True)
-                self._query_params = super(PyramidRequest, self).query_params
-        return self._query_params
+        try:
+            # Convert pyramid MultiDict to a normal dict with values as list
+            return dict(self.request.GET.dict_of_lists())
+        except Exception:
+            LOGGER.debug("couldn't get request.GET from the framework",
+                         exc_info=True)
+            return super(PyramidRequest, self).query_params
 
     @property
     def body(self):
@@ -48,37 +42,68 @@ class PyramidRequest(BaseWSGIRequest):
                          exc_info=True)
             return super(PyramidRequest, self).body
 
-    @property
+    @cached_property
     def form_params(self):
-        if self._form_params is None:
-            try:
-                # Convert pyramid MultiDict to a normal dict with values as list
-                form_params = {}
-                post_params = self.request.POST
-                for param_name in post_params:
-                    values = post_params.getall(param_name)
-                    # Ignore any non json serializable value as we don't know
-                    # how to process them (like cgi.FieldStorage)
-                    form_params[param_name] = list(
-                        filter(is_json_serializable, values)
-                    )
-                self._form_params = form_params
-            except Exception:
-                LOGGER.debug("couldn't get request.POST from framework",
-                             exc_info=True)
-                self._form_params = super(PyramidRequest, self).form_params
-        return self._form_params
+        try:
+            # Convert pyramid MultiDict to a normal dict with values as list
+            form_params = {}
+            post_params = self.request.POST
+            for param_name in post_params.keys():
+                values = post_params.getall(param_name)
+                # Ignore any non json serializable value as we don't know
+                # how to process them (like cgi.FieldStorage)
+                form_params[param_name] = list([value for value in values
+                                                if not isinstance(value, FieldStorage)])
+            return form_params
+        except Exception:
+            LOGGER.debug("couldn't get request.POST from framework",
+                         exc_info=True)
+            return super(PyramidRequest, self).form_params
 
-    @property
+    @cached_property
+    def filenames(self):
+        try:
+            return [
+                value.filename for value in self.request.POST.values()
+                if isinstance(value, FieldStorage) and value.filename
+            ]
+        except Exception:
+            LOGGER.debug("couldn't get request.POST from framework",
+                         exc_info=True)
+            return []
+
+    @cached_property
+    def combined_file_size(self):
+        try:
+            return sum([
+                int(value.headers["Content-Length"]) for value in self.request.POST.values()
+                if isinstance(value, FieldStorage) and "Content-Length" in value.headers
+            ])
+        except Exception:
+            LOGGER.debug("couldn't get request.POST from framework",
+                         exc_info=True)
+            return None
+
+    @cached_property
+    def files_field_names(self):
+        try:
+            return [
+                value.name for value in self.request.POST.values()
+                if isinstance(value, FieldStorage) and value.name
+            ]
+        except Exception:
+            LOGGER.debug("couldn't get request.POST from framework",
+                         exc_info=True)
+            return []
+
+    @cached_property
     def cookies_params(self):
-        if self._cookies_params is None:
-            try:
-                self._cookies_params = dict(self.request.cookies)
-            except Exception:
-                LOGGER.debug("couldn't get request.cookies from framework",
-                             exc_info=True)
-                self._cookies_params = super(PyramidRequest, self).cookies_params
-        return self._cookies_params
+        try:
+            return dict(self.request.cookies)
+        except Exception:
+            LOGGER.debug("couldn't get request.cookies from framework",
+                         exc_info=True)
+            return super(PyramidRequest, self).cookies_params
 
     @property
     def remote_addr(self):
@@ -128,16 +153,14 @@ class PyramidRequest(BaseWSGIRequest):
     def view_params(self):
         return self.request.matchdict
 
-    @property
+    @cached_property
     def json_params(self):
-        if self._json_params is None:
-            try:
-                self._json_params = self.request.json_body
-            except Exception:
-                LOGGER.debug("couldn't get request.json_body from the framework",
-                             exc_info=True)
-                self._json_params = super(PyramidRequest, self).json_params
-        return self._json_params
+        try:
+            return deepcopy(self.request.json_body)
+        except Exception:
+            LOGGER.debug("couldn't get request.json_body from the framework",
+                         exc_info=True)
+            return super(PyramidRequest, self).json_params
 
     @property
     def raw_headers(self):
@@ -157,9 +180,29 @@ class PyramidResponse(BaseResponse):
     def content_type(self):
         return self.response.headers.get("Content-Type")
 
-    @property
+    @cached_property
     def content_length(self):
         try:
             return int(self.response.content_length)
         except (ValueError, TypeError):
             return None
+
+    @cached_property
+    def headers_no_cookies(self):
+        result = {}
+        for header_name, value in self.response.items():
+            name = header_name.lower().replace("_", "-")
+            if name == "set-cookie":
+                continue
+            result[name] = value
+        return result
+
+    @cached_property
+    def body(self):
+        try:
+            cl = self.content_length
+            if cl is not None and cl < 4096 and self.response.has_body:
+                return self.response.text
+        except Exception:
+            LOGGER.debug("couldn't get response.text from the framework")
+        return None
