@@ -8,10 +8,10 @@
 """
 import logging
 
-from ..remote_exception import RemoteException
 from ..utils import HAS_ASYNCIO
 from .import_hook import ModuleFinder
 from .strategies import (
+    AWSLambdaStrategy,
     DBApi2Strategy,
     DjangoStrategy,
     FlaskStrategy,
@@ -25,6 +25,7 @@ from .strategies import (
 )
 
 STRATEGIES = {
+    "aws_lambda": AWSLambdaStrategy,
     "import_hook": ImportHookStrategy,
     "DBApi2": DBApi2Strategy,
     "django": DjangoStrategy,
@@ -88,36 +89,24 @@ class Instrumentation(object):
         """ Add a callback. The callback defines itself where it should
         hook to and the strategy use for hooking (set_attr or DBApi2)
         """
-        try:
-            strategy_class = self._get_strategy_class(callback.strategy)
+        strategy_class = self._get_strategy_class(callback.strategy)
 
-            # Get the strategy id
-            strategy_id = strategy_class.get_strategy_id(callback)
+        # Get the strategy id
+        strategy_id = strategy_class.get_strategy_id(callback)
 
-            # Check if we already have a strategy
-            if strategy_id in self.strategies:
-                strategy_instance = self.strategies[strategy_id]
-            else:
-                strategy_instance = strategy_class(
-                    strategy_id,
-                    self.observation_queue,
-                    self.queue,
-                    self.import_hook,
-                    self.before_hook_point,
-                )
-                self.strategies[strategy_id] = strategy_instance
-            strategy_instance.add_callback(callback)
-        except Exception:
-            # If the strategy fails to hook either at instantiation
-            # or hook method, catch the exception and log it
-            LOGGER.debug(
-                "Callback %r fails to hook", callback, exc_info=True
+        # Check if we already have a strategy
+        if strategy_id in self.strategies:
+            strategy_instance = self.strategies[strategy_id]
+        else:
+            strategy_instance = strategy_class(
+                strategy_id,
+                self.observation_queue,
+                self.queue,
+                self.import_hook,
+                self.before_hook_point,
             )
-
-            # And send the exception back to the backend
-            infos = {"callback": callback.__dict__}
-            remote_exception = RemoteException.from_exc_info(callback_payload=infos)
-            self.queue.put(remote_exception)
+            self.strategies[strategy_id] = strategy_instance
+        strategy_instance.add_callback(callback)
 
     def deinstrument(self, callback):
         """ Deactive instrumentation on the callback endpoint
