@@ -27,13 +27,15 @@ def get_aiohttp_middleware():
     from ..middlewares.aiohttp_middleware import AioHTTPMiddleware
 
     try:
-        __import__("aiohttp.web.middleware")  # type: ignore
+        from aiohttp import web
     except ImportError:
-        LOGGER.debug("New style middlewares are not supported")
-        return AioHTTPMiddleware.factory
-    else:
+        LOGGER.debug("Cannot import aiohttp.web", exc_info=True)
+        web = None
+    if hasattr(web, "middleware"):
         LOGGER.debug("New style middlewares are supported")
         return AioHTTPMiddleware.handle
+    LOGGER.debug("New style middlewares are not supported")
+    return AioHTTPMiddleware.factory
 
 
 class AioHTTPInstallStrategy(ImportHookStrategy):
