@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -7,10 +7,14 @@
 """ Logging module helpers
 """
 import logging
+import os
+import stat
 
-ROOT_LOGGER_NAME = 'sqreen'
-LOG_FORMAT = ('[%(levelname)s][%(asctime)s #%(process)d.%(threadName)s]'
-              ' %(name)s:%(lineno)s \t%(message)s')
+ROOT_LOGGER_NAME = "sqreen"
+LOG_FORMAT = (
+    "[%(levelname)s][%(asctime)s #%(process)d.%(threadName)s]"
+    " %(name)s:%(lineno)s \t%(message)s"
+)
 
 
 def configure_root_logger(log_level, log_location=None, root=ROOT_LOGGER_NAME):
@@ -40,7 +44,7 @@ def configure_root_logger(log_level, log_location=None, root=ROOT_LOGGER_NAME):
     if log_location is not None:
         try:
             filehandler = logging.FileHandler(log_location)
-
+            os.chmod(log_location, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP)
             filehandler.setFormatter(formatter)
             handlers.append(filehandler)
         except (OSError, IOError):
