@@ -6,13 +6,15 @@
 #
 """ Collect and aggregate metrics
 """
+import fnmatch
 import math
+import re
 from collections import Counter, defaultdict, namedtuple
 from datetime import timedelta
 from logging import getLogger
 
 from .exceptions import SqreenException
-from .utils import truncate_time
+from .utils import itervalues, truncate_time
 
 LOGGER = getLogger(__name__)
 
@@ -235,21 +237,20 @@ class MetricsStore(object):
         self.store = []
         self.metrics = {}
         self.aggregators = {}
+        self.definitions = {}
 
-    def register_metric(self, name, kind, period, **options):
+    def _register_metric(self, name, kind, period, **options):
         """ Register a new metric
         """
-        LOGGER.debug("Register metric '%s'", name)
-
         if name in self.metrics:
+            LOGGER.debug("Update existing metric %r", name)
+
             existing_metric = self.metrics[name]
             saved_metric_kind = existing_metric["kind"]
             if saved_metric_kind != kind:
-                err_msg = (
-                    "Metric '{}' has already been registered with kind {}"
-                )
                 raise AlreadyRegisteredMetric(
-                    err_msg.format(name, saved_metric_kind)
+                    "Metric {!r} has already been registered with kind {}"
+                    .format(name, saved_metric_kind)
                 )
 
             # Update the period
@@ -259,6 +260,8 @@ class MetricsStore(object):
                 existing_metric["options"] = options
                 self.metrics[name] = self.reset_metric(existing_metric)
         else:
+            LOGGER.debug("Creating new metric %r", name)
+
             if kind not in self.aggregators:
                 raise UnknownAggregator(
                     "Unknown aggregation kind: {}".format(kind)
@@ -266,17 +269,44 @@ class MetricsStore(object):
             metric = self._new_metric(kind, period, options)
             self.metrics[name] = self.reset_metric(metric)
 
-    def unregister_metric(self, name):
-        """ Unregister a metric
+    def register_metric_definition(self, name, kind, period, **options):
+        """ Register a new metric definition
         """
-        LOGGER.debug("Unregister metric %r", name)
-        self.metrics.pop(name, None)
+        if "*" in name:
+            LOGGER.debug("Register metric definition %r", name)
+            re_pattern = re.compile(fnmatch.translate(name))
+            for existing_name in list(self.metrics.keys()):
+                if re_pattern.match(existing_name):
+                    self._register_metric(existing_name, kind, period, **options)
+            if kind not in self.aggregators:
+                raise UnknownAggregator(
+                    "Unknown aggregation kind: {}".format(kind)
+                )
+            self.definitions[name] = (re_pattern, kind, period, options)
+        else:
+            self._register_metric(name, kind, period, **options)
+
+    def unregister_metric_definition(self, name):
+        """ Unregister a metric definition
+        """
+        if "*" in name:
+            LOGGER.debug("Unregister metric definition %r", name)
+
+            definition = self.definitions.pop(name, None)
+            if definition is not None:
+                for existing_name in list(self.metrics.keys()):
+                    if definition[0].match(existing_name):
+                        LOGGER.debug("Unregister metric %r", existing_name)
+                        self.metrics.pop(existing_name, None)
+        else:
+            LOGGER.debug("Unregister metric %r", name)
+            self.metrics.pop(name, None)
 
     def ensure_ext_perf_metric(self, name, period):
         if name in self.metrics:
             return
 
-        self.register_metric(name, 'ExtensionPerformance', period)
+        self._register_metric(name, 'ExtensionPerformance', period)
 
     @staticmethod
     def _new_metric(kind, period, options):
@@ -306,13 +336,14 @@ class MetricsStore(object):
         for aggregator in PRODUCTION_AGGREGATORS:
             self.register_aggregator(aggregator.name, aggregator)
 
-    def register_default_metrics(self):
-        """ Register production default metrics
+    def _new_metric_for_definition(self, name, at=None):
+        """ Create a new metric if it matches a metric definition.
         """
-
-        self.register_metric("sqreen_call_counts", "Sum", 60)
-        self.register_metric("whitelisted", "Sum", 60)
-        self.register_metric("request_overtime", "Sum", 60)
+        for pattern, kind, period, options in itervalues(self.definitions):
+            if pattern.match(name):
+                metric = self._new_metric(kind, period, options)
+                self.metrics[name] = self.reset_metric(metric, at)
+                return metric
 
     def update(self, name, at, key, value):
         """ Logic behind metric updating.
@@ -325,7 +356,9 @@ class MetricsStore(object):
         try:
             metric = self.metrics[name]
         except KeyError:
-            raise UnregisteredMetric("Unknown metric {}".format(name))
+            metric = self._new_metric_for_definition(name, at=at)
+            if metric is None:
+                raise UnregisteredMetric("Unknown metric {!r}".format(name))
 
         # Check if the metric should be published or not
         metric = self.check_metric_period_over(metric, name, at, False)
