@@ -15,11 +15,13 @@ from operator import itemgetter
 from sqreen.performance_metrics import PerformanceMetricsSettings
 
 from . import config
+from .remote_exception import RemoteException
+from .rules import ReactiveRuleCallback, RuleCallback
 from .rules_callbacks import cb_from_rule
 from .runtime_storage import runtime
 from .sdk.events import STACKTRACE_EVENTS
 from .signature import RSAVerifier
-from .utils import Mapping
+from .utils import Mapping, is_string
 
 LOGGER = logging.getLogger(__name__)
 
@@ -163,7 +165,7 @@ def _load_rules(runner, params=None, check_signature=True, storage=runtime):
             rulespack_id = rulespack.get("pack_id")
             rules = rulespack.get("rules", [])
 
-    if rules is None or len(rules) == 0 or rulespack_id is None:
+    if rules is None or rulespack_id is None:
         return None, []
 
     # Set the pack id on each rule
@@ -198,7 +200,7 @@ def _load_rules(runner, params=None, check_signature=True, storage=runtime):
 
         # Check if the rule has some metrics to register (even if not loaded)
         for metric in rule_dict.get("metrics", []):
-            runner.metrics_store.register_metric(**metric)
+            runner.metrics_store.register_metric_definition(**metric)
 
     return rulespack_id, callbacks
 
@@ -210,7 +212,28 @@ def _instrument_callbacks(runner, callbacks):
     instrumentation_remove(runner)
     LOGGER.info("Setup instrumentation")
     for callback in callbacks:
-        runner.instrumentation.add_callback(callback)
+        try:
+            if isinstance(callback, RuleCallback):
+                runner.instrumentation.add_callback(callback)
+            elif isinstance(callback, ReactiveRuleCallback):
+                if runner.reactive is not None:
+                    runner.reactive.add_callback(callback)
+                else:
+                    raise RuntimeError("reactive engine is disabled, cannot load {!r}".format(callback))
+            else:
+                raise RuntimeError("unknown callback {!r}".format(callback))
+        except Exception:
+            # If the callbacks fails to instantiate or hook method, catch the
+            # exception, log it and continue loading other callbacks.
+            LOGGER.debug(
+                "Callback %r fails to instantiate", callback, exc_info=True
+            )
+
+            # And send the exception back to the backend
+            infos = {"callback": getattr(callback, "__dict__", {})}
+            remote_exception = RemoteException.from_exc_info(callback_payload=infos)
+            runner.queue.put(remote_exception)
+
     runner.instrumentation.hook_all()
 
 
@@ -237,6 +260,8 @@ def instrumentation_remove(runner, params=None):
     """
     LOGGER.info("Remove current instrumentation")
     runner.instrumentation.deinstrument_all()
+    if runner.reactive is not None:
+        runner.reactive.remove_callbacks()
     return True
 
 
@@ -326,16 +351,12 @@ def features_change(runner, params):
     # Enable the signal client
     use_signals = params.get("use_signals")
     if use_signals is not None:
-        runner.set_signal_client(
-            params.get("batch_size", 0), params.get("max_staleness", 0),
-            use_signals=use_signals,
-        )
+        runner.session.use_signals = use_signals
 
     # Alter the deliverer if the batch_size is set
     if params.get("batch_size") is not None:
         runner.set_deliverer(
-            params["batch_size"], params.get("max_staleness", 0),
-            use_signals=use_signals or False,
+            params["batch_size"], params.get("max_staleness", 0)
         )
 
     new_features = copy(runner.features_get())
@@ -394,7 +415,7 @@ def tracing_enable(runner, params):
     tracing_identifier = params[0]
     tracing_scope_definition = params[1]
 
-    if not isinstance(tracing_identifier, str) or not isinstance(tracing_scope_definition, Mapping):
+    if not is_string(tracing_identifier) or not isinstance(tracing_scope_definition, Mapping):
         return {
             "status": False,
             "output": {"error": "wrong parameter types"},
