@@ -20,23 +20,48 @@ LOGGER = logging.getLogger(__name__)
 
 class BaseCallback(object):
     """ Base class for callbacks.
-
-    The hook_name is the path to the hook, it could be either a module, like
-    "package.module", or a specific class "package.module::Class".
-    The hook_name is the name of the function to hook_on, it's relative to
-    the hook_module, for example with a hook_module equal to
-    "package.module::Class" and a hook_path equal to "method", we will
-    hook on the method named "method" of a class named "Class" in the module
-    named "package.module"
     """
 
+    SECTIONS = ("request", "response", "params", "headers", "context")
+
     SUPPORTS_BUDGET = False  # override constants in subclasses supporting it
     INTERRUPTIBLE = True
 
-    def __init__(self, hook_module, hook_name, strategy=None):
-        self.hook_module = hook_module
-        self.hook_name = hook_name
-        self.strategy = strategy
+    def __init__(
+        self,
+        rule_name,
+        rulespack_id,
+        runner,
+        beta=None,
+        block=None,
+        test=None,
+        data=None,
+        metrics=None,
+        payload_sections=None,
+        priority=None,
+        attack_type=None,
+        storage=None,
+    ):
+        self.rule_name = rule_name
+        self.rulespack_id = rulespack_id
+        self.runner = runner
+
+        self.data = data or {}
+        self.beta = beta or False
+        self.block = block or False
+        self.test = test or False
+        self.metrics = metrics or []
+        self.storage = storage or runtime
+
+        self.attack_type = attack_type
+
+        # Payload sections.
+        if payload_sections is None:
+            payload_sections = self.SECTIONS
+        self.payload_sections = payload_sections
+
+        # Priority (lower number means higher priority)
+        self.priority = priority if priority is not None else 100
 
     @property
     def collaborative(self):
@@ -48,6 +73,129 @@ class BaseCallback(object):
         """Allow the callback to be skipped if request is overtime."""
         return self.INTERRUPTIBLE
 
+    @property
+    def whitelisted(self):
+        """ Return True if the callback should be skipped (whitelisted
+        request), False otherwise
+        """
+        return self.storage.get_whitelist_match() or False
+
+    @property
+    def performance_monitoring_enabled(self):
+        """ Return True if the performance monitoring is enabled for this rule.
+        """
+        if self.runner is not None:
+            return self.runner.performance_metrics and self.runner.performance_metrics_settings.enabled()
+        return False
+
+    def _apply_debug_log(self, methods):
+        if not LOGGER.isEnabledFor(logging.DEBUG):
+            return
+
+        for lifecycle in methods:
+            lifecycle_method = getattr(self, lifecycle, None)
+            if lifecycle_method is not None:
+                wrapped = debug_log_wrapper(lifecycle_method, lifecycle, self)
+                setattr(self, lifecycle, wrapped)
+
+    def _apply_perf_monitoring(self, methods):
+        # Only enable the performance monitoring if the feature flag is
+        if not self.performance_monitoring_enabled:
+            return
+
+        for lifecycle in methods:
+            lifecycle_method = getattr(self, lifecycle, None)
+            if lifecycle_method is not None:
+                metric_name = "sq.{}.{}".format(self.rule_name, lifecycle)
+                wrapped = perf_monitoring_wrapper(
+                    lifecycle_method, self.storage, metric_name)
+                setattr(self, lifecycle, wrapped)
+
+    def get_remaining_budget(self, callback_kwargs=None):
+        """ Get the remaining time we have for running this callback according
+        to the request budget if there is one. The budget can be overriden
+        with a special option in kwargs. Return None otherwise.
+        """
+        if callback_kwargs is not None:
+            override_budget = callback_kwargs.pop("__sqreen_override_budget", None)
+            if override_budget is not None:
+                return override_budget
+        if self.runner is not None and self.performance_monitoring_enabled:
+            budget = self.runner.budget
+            if budget is not None:
+                return budget - (self.storage.request_sq_time or 0)
+
+    def record_attack(self, infos=None, at=None):
+        """Record an attack."""
+        if at is None:
+            at = now()
+        payload = {
+            "infos": infos,
+            "rulespack_id": self.rulespack_id,
+            "rule_name": self.rule_name,
+            "beta": self.beta,
+            "block": self.block,
+            "test": self.test,
+            "attack_type": self.attack_type,
+            "time": at,
+        }
+        if "context" in self.payload_sections:
+            current_request = self.storage.get_current_request()
+            if current_request:
+                payload["backtrace"] = list(current_request.raw_caller)
+        LOGGER.debug("Observed attack %r", payload)
+        self.storage.observe("attacks", payload, self.payload_sections)
+
+    def record_observation(self, metric_name, key, value, at=None):
+        """Record a metric observation."""
+        if at is None:
+            at = now()
+        payload = (metric_name, at, key, value)
+        LOGGER.debug("Observed metric %r", payload)
+        self.storage.observe("observations", payload, report=False)
+
+    def exception_infos(self, infos={}):
+        """ Returns additional infos in case of exception
+        """
+        return {"rule_name": self.rule_name, "rulespack_id": self.rulespack_id}
+
+    def record_exception(self, exc, exc_info, stack=None, infos=None, at=None):
+        """Record an exception."""
+        if infos is None:
+            infos = {}
+        if at is None:
+            at = now()
+        # Try to recover some infos from the exception if it's a
+        # SqreenException.
+        try:
+            infos.update(exc.exception_infos())
+        except Exception:
+            pass
+        bt = raw_traceback_formatter(exc_info[2])
+        if stack is not None:
+            bt = traceback_formatter(stack) + bt
+        payload = {
+            "message": str(exc_info[1]),
+            "klass": exc_info[0].__name__,
+            "infos": infos,
+            "rule_name": self.rule_name,
+            "rulespack_id": self.rulespack_id,
+            "test": self.test,
+            "time": at,
+            "backtrace": bt,
+        }
+        LOGGER.debug("Observed exception %r", payload)
+        self.storage.observe("sqreen_exceptions", payload, self.payload_sections)
+
+    def record_overtime(self, lifecycle, at=None):
+        """Record an overtime execution."""
+        self.storage.record_overtime(
+            "{}.{}".format(self.rule_name, lifecycle), at=at)
+
+    def __repr__(self):
+        return "%s(rule_name=%r)" % (
+            self.__class__.__name__, self.rule_name)
+
 
 def check_condition_wrapper(wrapped, callback, condition, lifecycle):
     """ Wrapper that will check lifecycle method pre-condition before
@@ -73,6 +221,7 @@ def check_condition_wrapper(wrapped, callback, condition, lifecycle):
             "kwargs": kwargs,
             "data": callback.data,
             "rv": rv,
+            "request_store": storage.get_request_store(),
         }
 
         # Check the pre condition
@@ -153,8 +302,16 @@ def debug_log_wrapper(wrapped, lifecycle, callback):
 
 
 class RuleCallback(BaseCallback):
+    """Rule callback attached to a hookpoint.
 
-    SECTIONS = ("request", "response", "params", "headers", "context")
+    The hook_name is the path to the hook, it could be either a module, like
+    "package.module", or a specific class "package.module::Class".
+    The hook_name is the name of the function to hook_on, it's relative to
+    the hook_module, for example with a hook_module equal to
+    "package.module::Class" and a hook_path equal to "method", we will
+    hook on the method named "method" of a class named "Class" in the module
+    named "package.module"
+    """
 
     def __init__(
         self,
@@ -166,11 +323,11 @@ class RuleCallback(BaseCallback):
         beta=False,
         block=False,
         test=False,
-        strategy_cls=None,
+        strategy=None,
         data=None,
         conditions=None,
         callbacks=None,
-        call_count_interval=0,
+        call_count_interval=None,
         metrics=None,
         payload_sections=None,
         priority=None,
@@ -178,21 +335,25 @@ class RuleCallback(BaseCallback):
         attack_type=None,
         storage=runtime,
     ):
-        super(RuleCallback, self).__init__(
-            hook_module, hook_name, strategy_cls
-        )
-        self.rule_name = rule_name
-        self.rulespack_id = rulespack_id
+        self.hook_module = hook_module
+        self.hook_name = hook_name
+        self.strategy = strategy
 
-        if data is None:
-            data = {}
-        self.data = data
-        self.beta = beta
-        self.block = block
-        self.test = test
-        self.runner = runner
-        self.metrics = metrics
-        self.storage = storage
+        super(RuleCallback, self).__init__(
+            rule_name=rule_name,
+            rulespack_id=rulespack_id,
+            runner=runner,
+            beta=beta,
+            block=block,
+            test=test,
+            data=data,
+            metrics=metrics,
+            payload_sections=payload_sections,
+            priority=priority,
+            attack_type=attack_type,
+            storage=storage,
+        )
+        self.attack_render_info = attack_render_info
 
         if conditions is None:
             conditions = {}
@@ -201,28 +362,19 @@ class RuleCallback(BaseCallback):
         if callbacks is None:
             callbacks = {}
         self.callbacks = callbacks
-        self.attack_render_info = attack_render_info
-        self.attack_type = attack_type
 
         # Callbacks
-        self.call_count_interval = call_count_interval
+        self.call_count_interval = call_count_interval or 0
         self.call_counts = {"pre": 0, "post": 0, "failing": 0}
 
-        self._apply_debug_log()
-        self._apply_conditions()
-        self._apply_call_count()
-        self._apply_perf_monitoring()
-
-        # Payload sections.
-        if payload_sections is None:
-            payload_sections = self.SECTIONS
-        self.payload_sections = payload_sections
-
-        # Priority (lower number means higher priority)
-        self.priority = priority if priority is not None else 100
+        methods = self.lifecycle_methods
+        self._apply_debug_log(methods)
+        self._apply_conditions(methods)
+        self._apply_call_count(methods)
+        self._apply_perf_monitoring(methods)
 
     @classmethod
-    def from_rule_dict(cls, rule_dict, runner, storage=runtime):
+    def from_rule_dict(cls, rule_dict, runner, storage=None):
         """ Return a RuleCallback based on a rule dict
         """
         return cls(
@@ -230,50 +382,28 @@ class RuleCallback(BaseCallback):
             hook_name=rule_dict["hookpoint"]["method"],
             rule_name=rule_dict["name"],
             rulespack_id=rule_dict["rulespack_id"],
-            strategy_cls=rule_dict["hookpoint"].get("strategy", "import_hook"),
+            strategy=rule_dict["hookpoint"].get("strategy", "import_hook"),
             attack_render_info=rule_dict["hookpoint"].get("attack_render_info", {}),
-            beta=rule_dict.get("beta", False),
-            block=rule_dict.get("block", False),
-            test=rule_dict.get("test", False),
+            beta=rule_dict.get("beta"),
+            block=rule_dict.get("block"),
+            test=rule_dict.get("test"),
             data=rule_dict.get("data"),
             conditions=rule_dict.get("conditions"),
             callbacks=rule_dict.get("callbacks"),
-            call_count_interval=rule_dict.get("call_count_interval", 0),
+            call_count_interval=rule_dict.get("call_count_interval"),
             runner=runner,
-            metrics=rule_dict.get("metrics", []),
+            metrics=rule_dict.get("metrics"),
             payload_sections=rule_dict.get("payload"),
             priority=rule_dict.get("priority"),
             attack_type=rule_dict.get("attack_type"),
             storage=storage,
         )
 
-    @property
-    def performance_monitoring_enabled(self):
-        """ Return True if the performance monitoring is enabled for this rule.
-        """
-        if self.runner is not None:
-            return self.runner.performance_metrics and self.runner.performance_metrics_settings.enabled()
-        return False
-
-    def get_remaining_budget(self, callback_kwargs=None):
-        """ Get the remaining time we have for running this callback according
-        to the request budget if there is one. The budget can be overriden
-        with a special option in kwargs. Return None otherwise.
-        """
-        if callback_kwargs is not None:
-            override_budget = callback_kwargs.pop("__sqreen_override_budget", None)
-            if override_budget is not None:
-                return override_budget
-        if self.runner is not None and self.performance_monitoring_enabled:
-            budget = self.runner.budget
-            if budget is not None:
-                return budget - (self.storage.request_sq_time or 0)
-
-    def _apply_conditions(self):
+    def _apply_conditions(self, methods):
         """ Wrap each lifecycle methods if the Rule define them and if we have
         conditions for them.
         """
-        for lifecycle in LIFECYCLE_METHODS.values():
+        for lifecycle in methods:
             conditions = self.conditions.get(lifecycle)
             lifecycle_method = getattr(self, lifecycle, None)
             if lifecycle_method is not None and \
@@ -284,22 +414,12 @@ class RuleCallback(BaseCallback):
                     lifecycle_method, self, conditions, lifecycle)
                 setattr(self, lifecycle, wrapped)
 
-    def _apply_debug_log(self):
-        if not LOGGER.isEnabledFor(logging.DEBUG):
-            return
-
-        for lifecycle in LIFECYCLE_METHODS.values():
-            lifecycle_method = getattr(self, lifecycle, None)
-            if lifecycle_method is not None:
-                wrapped = debug_log_wrapper(lifecycle_method, lifecycle, self)
-                setattr(self, lifecycle, wrapped)
-
-    def _apply_call_count(self):
+    def _apply_call_count(self, methods):
         # Only count calls if call_count_interval is > 0
         if self.call_count_interval == 0:
             return
 
-        for lifecycle in LIFECYCLE_METHODS.values():
+        for lifecycle in methods:
             lifecycle_method = getattr(self, lifecycle, None)
             if lifecycle_method is not None:
 
@@ -313,98 +433,6 @@ class RuleCallback(BaseCallback):
                     lifecycle_method, self, lifecycle, observation_key)
                 setattr(self, lifecycle, wrapped)
 
-    def _apply_perf_monitoring(self):
-        # Only enable the performance monitoring if the feature flag is
-        if not self.performance_monitoring_enabled:
-            return
-
-        for lifecycle in LIFECYCLE_METHODS.values():
-            lifecycle_method = getattr(self, lifecycle, None)
-            if lifecycle_method is not None:
-
-                metric_name = "sq.{}.{}".format(self.rule_name, lifecycle)
-
-                # Register the performance metric if it does not exist yet
-                if self.runner is not None \
-                        and not self.runner.metrics_store.is_registered(metric_name):
-                    self.runner.metrics_store.register_metric(
-                        metric_name, "Binning",
-                        self.runner.performance_metrics_settings.period,
-                        base=self.runner.performance_metrics_settings.base,
-                        unit=self.runner.performance_metrics_settings.unit
-                    )
-
-                wrapped = perf_monitoring_wrapper(
-                    lifecycle_method, self.storage, metric_name)
-                setattr(self, lifecycle, wrapped)
-
-    def exception_infos(self, infos={}):
-        """ Returns additional infos in case of exception
-        """
-        return {"rule_name": self.rule_name, "rulespack_id": self.rulespack_id}
-
-    def record_attack(self, infos=None, at=None):
-        """Record an attack."""
-        if at is None:
-            at = now()
-        payload = {
-            "infos": infos,
-            "rulespack_id": self.rulespack_id,
-            "rule_name": self.rule_name,
-            "beta": self.beta,
-            "block": self.block,
-            "test": self.test,
-            "attack_type": self.attack_type,
-            "time": at,
-        }
-        if "context" in self.payload_sections:
-            current_request = self.storage.get_current_request()
-            if current_request:
-                payload["backtrace"] = list(current_request.raw_caller)
-        LOGGER.debug("Observed attack %r", payload)
-        self.storage.observe("attacks", payload, self.payload_sections)
-
-    def record_observation(self, metric_name, key, value, at=None):
-        """Record a metric observation."""
-        if at is None:
-            at = now()
-        payload = (metric_name, at, key, value)
-        LOGGER.debug("Observed metric %r", payload)
-        self.storage.observe("observations", payload, report=False)
-
-    def record_exception(self, exc, exc_info, stack=None, infos=None, at=None):
-        """Record an exception."""
-        if infos is None:
-            infos = {}
-        if at is None:
-            at = now()
-        # Try to recover some infos from the exception if it's a
-        # SqreenException.
-        try:
-            infos.update(exc.exception_infos())
-        except Exception:
-            pass
-        bt = raw_traceback_formatter(exc_info[2])
-        if stack is not None:
-            bt = traceback_formatter(stack) + bt
-        payload = {
-            "message": str(exc_info[1]),
-            "klass": exc_info[0].__name__,
-            "infos": infos,
-            "rule_name": self.rule_name,
-            "rulespack_id": self.rulespack_id,
-            "test": self.test,
-            "time": at,
-            "backtrace": bt,
-        }
-        LOGGER.debug("Observed exception %r", payload)
-        self.storage.observe("sqreen_exceptions", payload, self.payload_sections)
-
-    def record_overtime(self, lifecycle, at=None):
-        """Record an overtime execution."""
-        self.storage.record_overtime(
-            "{}.{}".format(self.rule_name, lifecycle), at=at)
-
     @property
     def lifecycle_methods(self):
         return [
@@ -413,14 +441,74 @@ class RuleCallback(BaseCallback):
             if hasattr(self, lifecycle)
         ]
 
-    @property
-    def whitelisted(self):
-        """ Return True if the callback should be skipped (whitelisted
-        request), False otherwise
-        """
-        return self.storage.get_whitelist_match() or False
-
     def __repr__(self):
         return "%s(rule_name=%r, hook_module=%r, hook_name=%r, strategy=%r)" % (
             self.__class__.__name__, self.rule_name, self.hook_module,
             self.hook_name, self.strategy)
+
+
+class ReactiveRuleCallback(BaseCallback):
+    """ Rule callbacks attached to data subscribed on the reactive engine.
+    """
+
+    def __init__(
+        self,
+        rule_name,
+        rulespack_id,
+        runner,
+        beta=False,
+        block=False,
+        test=False,
+        data=None,
+        authorized_addresses=None,
+        metrics=None,
+        payload_sections=None,
+        priority=None,
+        attack_type=None,
+        storage=runtime,
+    ):
+        super(ReactiveRuleCallback, self).__init__(
+            rule_name=rule_name,
+            rulespack_id=rulespack_id,
+            runner=runner,
+            beta=beta,
+            block=block,
+            test=test,
+            data=data,
+            metrics=metrics,
+            payload_sections=payload_sections,
+            priority=priority,
+            attack_type=attack_type,
+            storage=storage,
+        )
+        self.authorized_addresses = frozenset(authorized_addresses or [])
+        self.batch_addresses = self.authorized_addresses
+        self.group_addresses = frozenset()
+
+        self._apply_debug_log(["handler"])
+        self._apply_perf_monitoring(["handler"])
+
+    def handler(self, instance, args, kwargs, **options):
+        """ Handler called when addresses are available.
+        """
+        raise NotImplementedError
+
+    @classmethod
+    def from_rule_dict(cls, rule_dict, runner, storage=None):
+        """ Return a ReactiveRuleCallback based on a rule dict
+        """
+        return cls(
+            rule_name=rule_dict["name"],
+            rulespack_id=rule_dict["rulespack_id"],
+            beta=rule_dict.get("beta"),
+            block=rule_dict.get("block"),
+            test=rule_dict.get("test"),
+            data=rule_dict.get("data"),
+            authorized_addresses=rule_dict.get("reactive", {}).get("authorized_addresses"),
+            runner=runner,
+            metrics=rule_dict.get("metrics"),
+            payload_sections=rule_dict.get("payload"),
+            priority=rule_dict.get("priority"),
+            attack_type=rule_dict.get("attack_type"),
+            storage=storage,
+        )
