@@ -10,7 +10,7 @@ import logging
 
 from ..exceptions import InvalidSignature
 from ..remote_exception import RemoteException
-from ..rules import RuleCallback
+from ..rules import BaseCallback
 from ..runtime_storage import runtime
 from ..utils import HAS_ASYNCIO
 from .auth_metrics import (
@@ -21,6 +21,8 @@ from .auth_metrics import (
 )
 from .binding_accessor_counter import BindingAccessorCounter
 from .binding_accessor_matcher import BindingAccessorMatcherCallback
+from .binding_accessor_provide_data import BindingAccessorProvideData
+from .binding_accessor_return_value import BindingAccessorReturnValue
 from .count_http_codes import CountHTTPCodesCB
 from .count_http_codes_django import CountHTTPCodesCBDjango
 from .count_http_codes_flask import CountHTTPCodesCBFlask
@@ -36,6 +38,8 @@ from .headers_insert_flask import HeadersInsertCBFlask
 from .headers_insert_pyramid import HeadersInsertCBPyramid
 from .ip_blacklist import IPBlacklistCB
 from .not_found import NotFoundCB
+from .reactive_log import ReactiveLog
+from .reactive_waf import ReactiveWAF
 from .record_request import RecordRequest
 from .record_request_context import RecordRequestContext
 from .record_request_context_django import RecordRequestContextDjango
@@ -98,18 +102,23 @@ def cb_from_rule(rule, runner, rule_verifier=None, storage=runtime):
         LOGGER.critical("Can't verify signature of rule %s", rule["name"])
         raise InvalidSignature(rule["name"], rule["rulespack_id"])
 
+    hookpoint = rule.get("hookpoint", {})
     # Ask for native attack rendering
-    if "attack_render_info" in rule.get("hookpoint", {}):
+    if "attack_render_info" in hookpoint:
         callback_class_name = "ExtensionPassthruCB"
     # Use JSCB for internal_js_hook callback used in PHP
-    elif rule.get("hookpoint", {}).get("strategy") == "internal_js_hook":
+    elif hookpoint.get("strategy") == "internal_js_hook":
         callback_class_name = "JSCB"
     else:
-        callback_class_name = rule.get("hookpoint", {}).get("callback_class", "JSCB")
+        callback_class_name = hookpoint.get("callback_class")
+
+    # Default to JS callbacks for hookpoint
+    if callback_class_name is None:
+        callback_class_name = "JSCB"
 
     possible_subclass = globals().get(callback_class_name, None)
 
-    if possible_subclass and issubclass(possible_subclass, RuleCallback):
+    if possible_subclass and issubclass(possible_subclass, BaseCallback):
         try:
             return possible_subclass.from_rule_dict(rule, runner, storage=storage)
         except Exception:
@@ -136,6 +145,8 @@ __all__ = [
     "AuthMetricsCB",
     "BindingAccessorCounter",
     "BindingAccessorMatcherCallback",
+    "BindingAccessorProvideData",
+    "BindingAccessorReturnValue",
     "cb_from_rule",
     "CountHTTPCodesCB",
     "CountHTTPCodesCBAioHTTP",
@@ -157,6 +168,8 @@ __all__ = [
     "NotFoundCB",
     "NotFoundCBDjango",
     "NotFoundCBFlask",
+    "ReactiveLog",
+    "ReactiveWAF",
     "RecordRequest",
     "RecordRequestDjango",
     "RecordRequestFlask",
