@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -14,13 +14,13 @@ LOGGER = logging.getLogger(__name__)
 
 
 class HeadersInsertCBAioHTTP(BaseHeadersInsertCB):
-
-    def post(self, original, response, *args, **kwargs):
+    def post(self, instance, args, kwargs, result=None, **options):
         """Insert custom headers."""
         try:
+            response = result
             for header_name, header_value in self.headers.items():
                 response.headers[header_name] = header_value
         except Exception:
-            LOGGER.warning("An error occured", exc_info=True)
+            LOGGER.warning("An error occurred", exc_info=True)
 
         return {}
