@@ -9,25 +9,23 @@
 
 import logging
 import sys
-from datetime import datetime
 from time import time
 
-from sqreen.performance_metrics import PerformanceMetricsSettings
-
-from .actions import ACTION_STORE
+from .config import CONFIG
 from .deliverer import get_deliverer
 from .ip_radix import Radix
 from .list_filters import RequestPrefixListFilter
 from .metrics import UnregisteredMetric
+from .performance_metrics import PerformanceMetricsSettings
+from .samplers import TracingSampler
+from .utils import now
+from .workflow import Workflow
 
 if sys.version_info[0] < 3:
     from Queue import Empty, Queue, Full
 else:
     from queue import Empty, Queue, Full
 
-MAX_QUEUE_LENGTH = 100
-MAX_OBS_QUEUE_LENGTH = 1000
-
 LOGGER = logging.getLogger(__name__)
 
 
@@ -45,12 +43,13 @@ class CappedQueue(object):
     """ Capped queue with opiniatied methods
     """
 
-    def __init__(self, maxsize=None):
+    def __init__(self, maxsize=None, name=None):
         if maxsize is None:
-            maxsize = MAX_QUEUE_LENGTH
+            maxsize = CONFIG["MAX_QUEUE_LENGTH"]
 
         self.maxsize = maxsize
         self.queue = Queue(self.maxsize)
+        self.name = name or "queue"
 
     def get(self, timeout, block=True):
         """ Wait for up to timeout for an item to return and block while
@@ -64,10 +63,11 @@ class CappedQueue(object):
         return self.queue.get_nowait()
 
     def put(self, item):
-        """ Tries to put an item to the queue, if the queue is empty, pop an
+        """ Tries to put an item to the queue, if the queue is full, pop an
         item and try again
         """
         pushed = False
+        dropped = 0
         while pushed is False:
             try:
                 self.queue.put_nowait(item)
@@ -76,8 +76,11 @@ class CappedQueue(object):
                 try:
                     self.queue.get_nowait()
                     self.queue.task_done()
+                    dropped += 1
                 except Empty:
                     pass
+        if dropped:
+            LOGGER.error("%d items dropped from the %s", dropped, self.name)
 
     def clear(self):
         """ Clear all items in the queue
@@ -99,13 +102,15 @@ class RunnerSettings(object):
         self.ips_whitelist = Radix(None)
         self.paths_whitelist = RequestPrefixListFilter()
         self.debug_flag = None
+        self.tracing_identifier_prefix = None
 
     def set_ips_whitelist(self, addresses):
         """ Replace the current list of whitelisted IP networks
         """
-        self.ips_whitelist.clear()
+        new_ips_whitelist = Radix(None)
         for address in addresses:
-            self.ips_whitelist.insert(address, '/' in address)
+            new_ips_whitelist.insert(address, '/' in address)
+        self.ips_whitelist = new_ips_whitelist
 
     def ips_whitelist_match(self, address):
         """ Return the first matching IP network, or None if no match could be
@@ -147,6 +152,11 @@ class RunnerSettings(object):
         """
         return self.debug_flag
 
+    def set_tracing_identifier_prefix(self, tracing_identifier_prefix):
+        """ Set the current tracing idenftifier prefix.
+        """
+        self.tracing_identifier_prefix = tracing_identifier_prefix
+
 
 def process_initial_commands(initial_payload, runner):
     """ Process the initial non-standard login payload
@@ -165,7 +175,7 @@ def process_initial_commands(initial_payload, runner):
         runner.process_commands(commands)
 
     actions = initial_payload.get("actions")
-    ACTION_STORE.reload_from_dicts(actions or [])
+    runner.action_store.reload_from_dicts(actions or [])
 
 
 class BaseRunner(object):
@@ -193,10 +203,14 @@ class Runner(BaseRunner):
         remote_command,
         runtime_infos,
         instrumentation,
+        action_store,
         metrics_store,
         settings,
         initial_features=None,
         performance_metrics=True,
+        tracing_sampler=None,
+        interface_manager=None,
+        reactive=None,
     ):
         self.logger = logging.getLogger(
             "{}.{}".format(self.__module__, self.__class__.__name__)
@@ -207,11 +221,15 @@ class Runner(BaseRunner):
         self.remote_command = remote_command
         self.runtime_infos = runtime_infos
         self.instrumentation = instrumentation
+        self.action_store = action_store
         self.metrics_store = metrics_store
         self.settings = settings
         self.stop = False
         # Used to globally disable performance metrics in the agent from the daemon
         self.performance_metrics = performance_metrics
+        self.tracing_sampler = tracing_sampler
+        self.interface_manager = interface_manager
+        self.reactive = reactive
 
         # Save the time runner started for checking warmup period termination
         self.started = time()
@@ -224,15 +242,17 @@ class Runner(BaseRunner):
             "heartbeat_delay", self.HEARTBEAT_DELAY
         )
 
+        self.set_call_counts_metrics_period(
+            initial_features.get("call_counts_metrics_period", 60)
+        )
+
         self.set_performance_metrics_settings(
             PerformanceMetricsSettings.from_features(initial_features)
         )
 
         self.budget = None
 
-        self.whitelisted_metric = initial_features.get(
-            "whitelisted_metric", True
-        )
+        self.set_whitelisted_metric(initial_features.get("whitelisted_metric", True))
 
         self.last_heartbeat_request = 0
 
@@ -244,6 +264,9 @@ class Runner(BaseRunner):
         # Next things to send on next heartbeat
         self.next_commands_results = {}
 
+        if self.interface_manager is not None:
+            self.interface_manager.register(Workflow(self))
+
     def run(self):
         """ Infinite loop
         """
@@ -311,13 +334,18 @@ class Runner(BaseRunner):
         """ Do an heartbeat, publish finished metrics from MetricsStore and
         past commands results
         """
-        metrics = self.metrics_store.get_data_to_publish(
-            datetime.utcnow(), force_finalize=False
-        )
+        use_signals = self.session.use_signals
         payload = {
             "command_results": self.next_commands_results,
-            "metrics": metrics,
         }
+        if not use_signals:
+            metrics = self.metrics_store.get_data_to_publish(
+                now(), force_finalize=False)
+            payload["metrics"] = [
+                {k: v for k, v in metric.items() if k in ("name", "observation", "start", "finish")}
+                for metric in metrics
+            ]
+
         res = self.session.heartbeat(payload)
         self.last_heartbeat_request = time()
 
@@ -327,6 +355,9 @@ class Runner(BaseRunner):
         if res is not None:
             self.process_commands(res["commands"])
 
+        if use_signals:
+            self.publish_metrics(force_finalize=False)
+
     def aggregate_observations(self):
         """ Empty the observation queue and update the metric store
         with the observations in the queue.
@@ -347,12 +378,13 @@ class Runner(BaseRunner):
             LOGGER.debug("Observations for unregistered metrics: %s",
                          ", ".join(unregistered_metrics))
 
-    def publish_metrics(self):
+    def publish_metrics(self, force_finalize=True):
         """ Publish finished metrics from MetricsStore
         """
-        self.session.post_metrics(
-            self.metrics_store.get_data_to_publish(datetime.utcnow())
-        )
+        metrics = self.metrics_store.get_data_to_publish(
+            now(), force_finalize=force_finalize)
+        if metrics:
+            self.deliverer.post_metrics(metrics)
 
     def _should_do_heartbeat(self):
         """ Check if we should send an heartbeat because the delay is overdue
@@ -386,6 +418,7 @@ class Runner(BaseRunner):
         res = {
             "heartbeat_delay": self.heartbeat_delay,
             "batch_size": self.deliverer.batch_size,
+            "use_signals": self.session.use_signals,
             "max_staleness": self.deliverer.original_max_staleness,
             "call_counts_metrics_period": self.metrics_store.get_metric_period(
                 "sqreen_call_counts"
@@ -408,14 +441,14 @@ class Runner(BaseRunner):
 
         if budget is not None and self.performance_metrics_settings.enabled():
             LOGGER.info("Updated performance cap budget to %0.2fms", budget)
-            self.metrics_store.register_metric(
+            self.metrics_store.register_metric_definition(
                 "request_overbudget_cb", "Sum",
                 self.performance_metrics_settings.overtime_period
             )
             self.budget = budget
         else:
             LOGGER.info("Disabled performance cap")
-            self.metrics_store.unregister_metric("request_overbudget_cb")
+            self.metrics_store.unregister_metric_definition("request_overbudget_cb")
             self.budget = None
 
     def set_performance_metrics_settings(self, new_perf_met_set):
@@ -424,46 +457,45 @@ class Runner(BaseRunner):
             return
         if self.performance_metrics_settings.enabled():
             LOGGER.info("Updated performance monitoring settings")
-            self.metrics_store.register_metric(
+            self.metrics_store.register_metric_definition(
                 "req", "Binning", self.performance_metrics_settings.period,
                 base=self.performance_metrics_settings.base,
                 unit=self.performance_metrics_settings.unit
             )
-            self.metrics_store.register_metric(
+            self.metrics_store.register_metric_definition(
                 "sq", "Binning", self.performance_metrics_settings.period,
                 base=self.performance_metrics_settings.base,
                 unit=self.performance_metrics_settings.unit
             )
-            self.metrics_store.register_metric(
+            self.metrics_store.register_metric_definition(
                 "pct", "Binning", self.performance_metrics_settings.period,
                 base=self.performance_metrics_settings.pct_base,
                 unit=self.performance_metrics_settings.pct_unit
             )
-            for metric_name in self.metrics_store.list_metrics():
-                # Reload performance monitoring for rules
-                if metric_name.startswith("sq."):
-                    self.metrics_store.register_metric(
-                        metric_name, "Binning",
-                        self.performance_metrics_settings.period,
-                        base=self.performance_metrics_settings.base,
-                        unit=self.performance_metrics_settings.unit
-                    )
+            self.metrics_store.register_metric_definition(
+                "sq.*", "Binning",
+                self.performance_metrics_settings.period,
+                base=self.performance_metrics_settings.base,
+                unit=self.performance_metrics_settings.unit
+            )
         else:
             LOGGER.info("Disabled performance monitoring")
-            self.metrics_store.unregister_metric("req")
-            self.metrics_store.unregister_metric("sq")
-            self.metrics_store.unregister_metric("pct")
-            for metric_name in self.metrics_store.list_metrics():
-                if metric_name.startswith("sq."):
-                    self.metrics_store.unregister_metric(metric_name)
+            self.metrics_store.unregister_metric_definition("req")
+            self.metrics_store.unregister_metric_definition("sq")
+            self.metrics_store.unregister_metric_definition("pct")
+            self.metrics_store.unregister_metric_definition("sq.*")
 
     def set_call_counts_metrics_period(self, call_counts_metrics_period):
-        self.metrics_store.register_metric(
+        self.metrics_store.register_metric_definition(
             "sqreen_call_counts", "Sum", call_counts_metrics_period
         )
 
     def set_whitelisted_metric(self, enabled):
         self.whitelisted_metric = enabled
+        if enabled:
+            self.metrics_store.register_metric_definition("whitelisted", "Sum", 60)
+        else:
+            self.metrics_store.unregister_metric_definition("whitelisted")
 
     def set_deliverer(self, batch_size, max_staleness):
         # Drain current deliverer
@@ -483,3 +515,15 @@ class Runner(BaseRunner):
         """
         self.logger.debug("Set paths whitelist on settings %r", self.settings)
         self.settings.set_paths_whitelist(paths)
+
+    def set_tracing_configuration(self, tracing_identifier_prefix, scope_definition):
+        """ Update the tracing configuration
+        """
+        self.logger.debug("Set the tracing identifier prefix to %r", tracing_identifier_prefix)
+        self.settings.set_tracing_identifier_prefix(tracing_identifier_prefix)
+        if self.tracing_sampler is None:
+            self.logger.debug("Creating a new tracing sampler with scope definition %r", scope_definition)
+            self.tracing_sampler = TracingSampler(scope_definition)
+        else:
+            self.logger.debug("Updating tracing sampler scope definition %r", scope_definition)
+            self.tracing_sampler.update_scope_definition(scope_definition)
