@@ -38,11 +38,10 @@ from .instrumentation import Instrumentation
 from .interface_manager import InterfaceManager
 from .log import configure_root_logger
 from .metrics import MetricsStore
+from .reactive import Engine as Reactive
 from .remote_command import RemoteCommand
 from .remote_exception import RemoteException
 from .runner import (
-    MAX_OBS_QUEUE_LENGTH,
-    MAX_QUEUE_LENGTH,
     CappedQueue,
     Runner,
     RunnerSettings,
@@ -55,7 +54,6 @@ from .runtime_infos import (
     get_process_cmdline,
 )
 from .session import InvalidSession, InvalidToken, Session
-from .signal_client import get_signal_client
 from .utils import (
     HAS_TYPING,
     configure_newrelics_ignore_exception,
@@ -195,6 +193,7 @@ def get_proxy_url():
 
 
 def post_app_sqreen_exception(data):
+    # type: (RemoteException) -> None
     """Post a Sqreen exception happening at application level.
     This function does not need a session. It always uses the legacy backend URL
     until all customers are migrated.
@@ -384,31 +383,25 @@ def get_runner(
     instrumentation,
     settings,
     interface_manager,
+    reactive,
 ):
     initial_features = get_initial_features(
         config["INITIAL_FEATURES"], login_result.get("features", {})
     )
 
     # Get a signal client if the feature is enabled
-    use_signals = initial_features.get("use_signals", False)
-    signal_client = get_signal_client(
-        session=session,
-        batch_size=initial_features.get("batch_size", 0),
-        max_staleness=initial_features.get("max_staleness", 0)
-    ) if use_signals else None
+    session.use_signals = initial_features.get("use_signals", False)
 
     # Get the right deliverer according to initial features
     deliverer = get_deliverer(
         initial_features.get("batch_size", 0),
         initial_features.get("max_staleness", 0),
         session,
-        signal_client=signal_client,
     )
     remote_command = RemoteCommand.with_production_commands()
     action_store = ActionStore()
     metrics_store = MetricsStore()
     metrics_store.register_production_aggregators()
-    metrics_store.register_default_metrics()
 
     # Setup the runner
     runner = Runner(
@@ -423,8 +416,8 @@ def get_runner(
         metrics_store,
         settings,
         initial_features,
-        signal_client=signal_client,
         interface_manager=interface_manager,
+        reactive=reactive,
     )
     return runner
 
@@ -436,11 +429,12 @@ class RunnerThread(Thread):
     name = "SqreenRunnerThread"
 
     def __init__(self, runtime_infos, queue=None, observations_queue=None,
-                 instrumentation=None, settings=None, interface_manager=None):
+                 instrumentation=None, settings=None, interface_manager=None,
+                 reactive=None):
         super(RunnerThread, self).__init__()
         self.daemon = True
-        self.queue = queue or CappedQueue(MAX_QUEUE_LENGTH)
-        self.observations_queue = observations_queue or CappedQueue(MAX_OBS_QUEUE_LENGTH)
+        self.queue = queue or CappedQueue(CONFIG["MAX_QUEUE_LENGTH"])
+        self.observations_queue = observations_queue or CappedQueue(CONFIG["MAX_OBS_QUEUE_LENGTH"], name="observations queue")
         self.unblock_main_thread = Event()
         self.runtime_infos = runtime_infos
         self.instrumentation = instrumentation or Instrumentation(
@@ -450,6 +444,7 @@ class RunnerThread(Thread):
         if CONFIG["BETA_STATIC_INSTRUMENTATION"]:
             interface_manager = interface_manager or InterfaceManager()
         self.interface_manager = interface_manager
+        self.reactive = reactive or Reactive(self.observations_queue, self.queue)
         self.runner = None
         self.started = False
         self.pid = None
@@ -564,6 +559,7 @@ class RunnerThread(Thread):
                     self.instrumentation,
                     self.settings,
                     self.interface_manager,
+                    self.reactive,
                 )
 
                 # Process the initial commands
@@ -586,7 +582,7 @@ class RunnerThread(Thread):
                         )
                         try:
                             session.post_sqreen_exception(
-                                RemoteException.from_exc_info().to_dict()
+                                RemoteException.from_exc_info()
                             )
                         except HTTPError:
                             pass
@@ -611,9 +607,7 @@ class RunnerThread(Thread):
 
                 if session is not None and session.is_connected():
                     try:
-                        session.post_sqreen_exception(
-                            remote_exception.to_dict()
-                        )
+                        session.post_sqreen_exception(remote_exception)
                         session.logout()
                     except HTTPError:
                         LOGGER.exception("An HTTP error occurred while logout")
@@ -621,7 +615,7 @@ class RunnerThread(Thread):
                         LOGGER.exception("Exception while logout")
                         return
                 else:
-                    post_app_sqreen_exception(remote_exception.to_dict())
+                    post_app_sqreen_exception(remote_exception)
 
                 try:
                     self.instrumentation.deinstrument_all()
@@ -677,6 +671,7 @@ class RunnerThread(Thread):
             instrumentation=self.instrumentation,
             settings=self.settings,
             interface_manager=self.interface_manager,
+            reactive=self.reactive,
         )
         nkwargs.update(kwargs)
         return self.__class__(RuntimeInfos().all(), **nkwargs)
