@@ -56,6 +56,9 @@ if sys.version_info[0] < 3:
     STRING_CLASS = str
     UNICODE_CLASS = unicode  # noqa
 
+    def viewkeys(d, **kwargs):
+        return d.viewkeys(**kwargs)
+
     def iterkeys(d, **kwargs):
         return d.iterkeys(**kwargs)
 
@@ -88,9 +91,11 @@ else:
     STRING_CLASS = str
     UNICODE_CLASS = str
 
-    def iterkeys(d, **kwargs):
+    def viewkeys(d, **kwargs):
         return d.keys(**kwargs)
 
+    iterkeys = viewkeys
+
     def itervalues(d, **kwargs):
         return d.values(**kwargs)
 
@@ -510,3 +515,54 @@ def datetime_from_isoformat(date_string):
         time_components = [0, 0, 0, 0, None]
 
     return datetime.datetime(*(date_components + time_components))
+
+
+missing = object()
+
+
+class cached_property(property):
+    """From werkzeug source code:
+
+    A decorator that converts a function into a lazy property.  The
+    function wrapped is called the first time to retrieve the result
+    and then that calculated result is used the next time you access
+    the value::
+
+        class Foo(object):
+
+            @cached_property
+            def foo(self):
+                # calculate something important here
+                return 42
+
+    The class has to have a `__dict__` in order for this property to
+    work.
+    """
+
+    # implementation detail: A subclass of python's builtin property
+    # decorator, we override __get__ to check for a cached value. If one
+    # chooses to invoke __get__ by hand the property will still work as
+    # expected because the lookup logic is replicated in __get__ for
+    # manual invocation.
+
+    def __init__(self, func, name=None, doc=None):
+        super(cached_property, self).__init__()
+        self.__name__ = name or func.__name__
+        self.__module__ = func.__module__
+        self.__doc__ = doc or func.__doc__
+        self.func = func
+
+    def __set__(self, obj, value):
+        obj.__dict__[self.__name__] = value
+
+    def __get__(self, obj, type=None):
+        if obj is None:
+            return self
+        value = obj.__dict__.get(self.__name__, missing)
+        if value is missing:
+            value = self.func(obj)
+            if value is missing:
+                # don't cache when the function returns missing.
+                return None
+            obj.__dict__[self.__name__] = value
+        return value
