@@ -41,10 +41,10 @@ class Workflow:
     def maybe_record_transport(self, scope, transport, at=None):
         # type: (str, Transport, Optional[datetime]) -> bool
         sampler = self.runner.tracing_sampler
-        signal_client = self.runner.signal_client
         tracing_identifier_prefix = self.runner.settings.tracing_identifier_prefix
+        use_signals = self.runner.session.use_signals
         if tracing_identifier_prefix is None or sampler is None \
-                or signal_client is None:
+                or not use_signals:
             return False
         if at is None:
             at = now()
@@ -70,8 +70,8 @@ class Workflow:
             trigger=decision.attrs["trigger"],
             time=at,
         )
-        LOGGER.debug("Sending signal %r", signal)
-        signal_client.signal(**signal)
+        LOGGER.debug("Queuing signal %r", signal)
+        self.runner.queue.put(signal)
         return True
 
     def new_data_in_pre(self, context, transport):  # type: (Context, Transport) -> bool
